@@ -1,5 +1,13 @@
-;;; Compiled by f2cl version 2.0 beta Date: 2006/12/21 03:42:11 
-;;; Using Lisp CMU Common Lisp CVS Head 2006-12-02 00:15:46 (19D)
+;;; Compiled by f2cl version:
+;;; ("f2cl1.l,v 1.215 2009/04/07 22:05:21 rtoy Exp $"
+;;;  "f2cl2.l,v 1.37 2008/02/22 22:19:33 rtoy Exp $"
+;;;  "f2cl3.l,v 1.6 2008/02/22 22:19:33 rtoy Exp $"
+;;;  "f2cl4.l,v 1.7 2008/02/22 22:19:34 rtoy Exp $"
+;;;  "f2cl5.l,v 1.200 2009/01/19 02:38:17 rtoy Exp $"
+;;;  "f2cl6.l,v 1.48 2008/08/24 00:56:27 rtoy Exp $"
+;;;  "macros.l,v 1.112 2009/01/08 12:57:19 rtoy Exp $")
+
+;;; Using Lisp CMU Common Lisp 19f (19F)
 ;;; 
 ;;; Options: ((:prune-labels nil) (:auto-save t) (:relaxed-array-decls t)
 ;;;           (:coerce-assigns :as-needed) (:array-type ':array)
@@ -10,9 +18,9 @@
 
 
 (let* ((zero 0.0))
-  (declare (type (double-float 0.0 0.0) zero))
-  (defun dlasq6 (i0 n0 z pp dmin f2cl-lib:dmin1 dmin2 dn dnm1 dnm2)
-    (declare (type (double-float) dnm2 dnm1 dn dmin2 f2cl-lib:dmin1 dmin)
+  (declare (type (double-float 0.0 0.0) zero) (ignorable zero))
+  (defun dlasq6 (i0 n0 z pp dmin dmin1$ dmin2 dn dnm1 dnm2)
+    (declare (type (double-float) dnm2 dnm1 dn dmin2 dmin1$ dmin)
              (type (array double-float (*)) z)
              (type (f2cl-lib:integer4) pp n0 i0))
     (f2cl-lib:with-multi-array-data
@@ -301,7 +309,7 @@
                                       ((1 *))
                                       z-%offset%))))))
         (setf dmin (min dmin dnm1))
-        (setf f2cl-lib:dmin1 dmin)
+        (setf dmin1$ dmin)
         (setf j4 (f2cl-lib:int-add j4 4))
         (setf j4p2
                 (f2cl-lib:int-sub (f2cl-lib:int-add j4 (f2cl-lib:int-mul 2 pp))
@@ -385,8 +393,7 @@
                 emin)
         (go end_label)
        end_label
-        (return
-         (values nil nil nil nil dmin f2cl-lib:dmin1 dmin2 dn dnm1 dnm2))))))
+        (return (values nil nil nil nil dmin dmin1$ dmin2 dn dnm1 dnm2))))))
 
 (in-package #-gcl #:cl-user #+gcl "CL-USER")
 #+#.(cl:if (cl:find-package '#:f2cl) '(and) '(or))
@@ -399,7 +406,7 @@
                         (double-float) (double-float) (double-float)
                         (double-float) (double-float) (double-float))
            :return-values '(nil nil nil nil fortran-to-lisp::dmin
-                            fortran-to-lisp::dmin1 fortran-to-lisp::dmin2
+                            fortran-to-lisp::dmin1$ fortran-to-lisp::dmin2
                             fortran-to-lisp::dn fortran-to-lisp::dnm1
                             fortran-to-lisp::dnm2)
            :calls '(fortran-to-lisp::dlamch))))
