@@ -14,11 +14,15 @@
 
 (declare-top (special $exptsubst $linechar $nolabels $inflag $piece $dispflag
 		      $gradefs $props $dependencies derivflag derivlist
-		      $linenum $partswitch linelable nn* dn* islinp
-		      $powerdisp atvars atp $errexp $derivsubst $dotdistrib
+		      $linenum $partswitch linelable nn* dn*
+		      $powerdisp atvars $errexp $derivsubst $dotdistrib
 		      $opsubst $subnumsimp $transrun in-p substp $sqrtdispflag
 		      $pfeformat dummy-variable-operators))
 
+(defvar *islinp* nil) ; When T, sdiff is called from the function islinear.
+(defvar *atp* nil)    ; When T, prevents substitution from applying to vars 
+                      ; bound by %sum, %product, %integrate, %limit
+
 ;; op and opr properties
 
 (defvar *opr-table* (make-hash-table :test #'equal))
@@ -53,15 +57,12 @@
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'op))
       '((mqapply $subvar) (bigfloat $bfloat)))
 
-
 (setq $exptsubst nil
       $partswitch nil
       $inflag nil
       $gradefs '((mlist simp))
       $dependencies '((mlist simp))
       atvars '($@1 $@2 $@3 $@4)
-      atp nil
-      islinp nil
       lnorecurse nil
       $derivsubst nil
       timesp nil
@@ -73,7 +74,6 @@
   "If TRUE allows DIFF(X~Y,T) to work where ~ is defined in
 	  SHARE;VECT where VECT_CROSS is set to TRUE.")
 
-
 (defmfun $substitute (old new &optional (expr nil three-arg?))
   (cond (three-arg? (maxima-substitute old new expr))
 	(t
@@ -86,6 +86,53 @@
 			((null l) z)
 		      (setq z ($substitute (car l) z)))))))))
 
+;; Define an alias $psubst and a reversealias for $psubstitute
+(defprop $psubst $psubstitute alias)
+(defprop $psubstitute $psubst reversealias)
+
+;; $psubstitute is similar to $substitute. In distinction from $substitute
+;; the function $psubstitute does parallel substitution, if the first argument
+;; is a list of equations.
+(defun $psubstitute (old new &optional (expr nil three-arg?))
+  (cond (three-arg? (maxima-substitute old new expr))
+        (t
+         (let ((l old) (z new))
+           (cond ((and ($listp l)
+                       ($listp (cadr l))
+                       (null (cddr l)))
+                  ;; A nested list.
+                  ($psubstitute (cadr l) z))
+                 ((and ($listp l)
+                       (eq (caar (cadr l)) 'mequal)
+                       (null (cddr l)))
+                  ;; A list with one equation.
+                  ($psubstitute (cadr l) z))
+                 ((notloreq l) (improper-arg-err l '$psubstitute))
+                 ((eq (caar l) 'mequal)
+                  ;; Do a substitution for one equation.
+                  (maxima-substitute (caddr l) (cadr l) z))
+                 (t
+                  ;; We have a list of equations. We do parallel subsitution.
+                  (let (gensymbol genlist eqn ($simp nil))
+                    ;; At first substitute a gensym for the expressions of
+                    ;; the left hand side of the equations.
+                    (do ((l (cdr l) (cdr l)))
+                        ((null l) z)
+                      (setq eqn (car l))
+                      (when (not (eq 'mequal (caar eqn)))
+                        (improper-arg-err old '$substitute))
+                      (setq gensymbol (gensym))
+                      ;; Store the gensym and the new expression into a list.
+                      (push (cons gensymbol (caddr eqn)) genlist)
+                      ;; Substitute a gensym for the old expression.
+                      (setq z (maxima-substitute gensymbol (cadr eqn) z)))
+                      ;; Substitute the new expressions for the gensyms.
+                      (do ((l genlist (cdr l)))
+                          ((null l)
+                           ;; Resimplify the result.
+                           (let (($simp t)) (resimplify z)))
+                        (setq z (maxima-substitute (cdar l) (caar l) z))))))))))
+
 (declare-top (special x y oprx opry negxpty timesp))
 
 (defmfun maxima-substitute (x y z) ; The args to SUBSTITUTE are assumed to be simplified.
@@ -117,7 +164,7 @@
 
 ;;Remainder of page is update from F302 --gsb
 
-;;Used only in COMM2 (AT), and below.
+;;Used in COMM2 (AT), limit, and below.
 (defvar dummy-variable-operators '(%product %sum %laplace %integrate %limit %at))
 
 (defun subst1 (z)			; Y is an atom
@@ -126,7 +173,7 @@
 	((eq (caar z) 'bigfloat) z)
 	((and (eq (caar z) 'rat) (or (equal y (cadr z)) (equal y (caddr z))))
 	 (div (subst1 (cadr z)) (subst1 (caddr z))))
-	((at-substp z) z)
+	((at-substp z) (subst-except-second-arg x y z))
 	((and (eq y t) (eq (caar z) 'mcond))
 	 (list (cons (caar z) nil) (subst1 (cadr z)) (subst1 (caddr z))
 	       (cadddr z) (subst1 (car (cddddr z)))))
@@ -145,8 +192,7 @@
 			     ($subvarp x))
 			 (let ((substp 'mqapply))
 			   (subst0 (list* '(mqapply) x margs) z))
-			 (merror "Attempt to MAXIMA-SUBSTITUTE ~M for ~M in ~M~
-			   ~%Illegal substitution for operator of expression" x y z))
+			 (merror (intl:gettext "subst: cannot substitute ~M for operator ~M in expression ~M") x y z))
 		     (subst0 (cons (cons oprx nil) margs) z))
 		 (subst0 (cons (cons (caar z) nil) margs) z))))))
 
@@ -154,7 +200,7 @@
   (let (newexpt)
     (cond ((atom z) z)
 	  ((specrepp z) (subst2 (specdisrep z)))
-	  ((and atp (member (caar z) '(%derivative %laplace) :test #'eq)) z)
+	  ((and *atp* (member (caar z) '(%derivative %laplace) :test #'eq)) z)
 	  ((at-substp z) z)
 	  ((alike1 y z) x)
 	  ((and timesp (eq (caar z) 'mtimes) (alike1 y (setq z (nformat z)))) x)
@@ -169,6 +215,23 @@
 		   (t (cons (cons (caar z) nil) (cons x (cdr tail)))))))
 	  (t (recur-apply #'subst2 z)))))
 
+;; replace y with x in z, but leave z's second arg unchanged.
+;; This is for cases like at(integrate(x, x, a, b), [x=3])
+;; where second arg of integrate binds a new variable x,
+;; and we do not wish to subst 3 for x inside integrand.
+(defun subst-except-second-arg (x y z)
+  (cond 
+    ((member (caar z) '(%integrate %sum %product %limit))
+     (append 
+       (list (car z)
+             (if (eq y (third z))     ; if (third z) is new var that shadows y
+                 (second z)           ; leave (second z) unchanged
+                 (subst1 (second z))) ; otherwise replace y with x in (second z)
+             (third z))               ; never change integration var
+       (mapcar (lambda (z) (subst1 z)); do subst in limits of integral
+               (cdddr z))))
+    (t z)))
+
 (declare-top (unspecial x y oprx opry negxpty timesp))
 
 (defmfun subst0 (new old)
@@ -220,8 +283,10 @@
 
 ;;This probably should be a subst or macro.
 (defun at-substp (z)
-  (and atp (or (member (caar z) '(%derivative %del) :test #'eq)
-	       (member (caar z) dummy-variable-operators :test #'eq))))
+  (and *atp*
+       (or (member (caar z) '(%derivative %del) :test #'eq)
+           (member (caar z) dummy-variable-operators :test #'eq))))
+
 (defmfun recur-apply (fun e)
   (cond ((eq (caar e) 'bigfloat) e)
 	((specrepp e) (funcall fun (specdisrep e)))
@@ -233,12 +298,13 @@
 
 (defmfun $depends (&rest args)
   (when (oddp (length args))
-    (merror "`depends' takes an even number of arguments."))
+    (merror (intl:gettext "depends: number of arguments must be even.")))
   (do ((args args (cddr args))
        (l))
       ((null args) (i-$dependencies (nreverse l)))
     (if ($listp (first args))
-	(mapc #'(lambda (e) (push (depends1 e (second args)) l)) (cdr (first args)))
+	(mapc #'(lambda (e) (push (depends1 e (second args)) l))
+	      (cdr (first args)))
 	(push (depends1 (first args) (second args)) l))))
 
 (defun depends1 (x y)
@@ -248,40 +314,56 @@
 (defmspec $dependencies (form)
   (i-$dependencies (cdr form)))
 
-(defmfun i-$dependencies (l)
+(defun i-$dependencies (l &aux res)
   (dolist (z l)
-    (cond ((atom z) (merror "Wrong format.  Try F(X)."))
-	  ((or (eq (caar z) 'mqapply) (member 'array (cdar z) :test #'eq))
-	   (merror "Improper form for `depends':~%~M" z))
-	  (t (let ((y (mget (caar z) 'depends)))
-	       (mputprop (caar z)
-			 (setq y (union* (reverse (cdr z)) y))
-			 'depends)
-	       (unless (cdr $dependencies)
-		 (setq $dependencies (copy-list '((mlist simp)))))
-	       (add2lnc (cons (cons (caar z) nil) y) $dependencies)))))
-  (cons '(mlist simp) l))
+    (cond
+      ((atom z)
+       (merror
+         (intl:gettext
+           "depends: argument must be a non-atomic expression; found ~M") z))
+      (t
+       (do ((zz z (cdr zz))
+            (y nil))
+           ((null zz)
+            (mputprop (caar z) (setq y (reverse y)) 'depends)
+            (setq res (push (cons (ncons (caar z)) y) res))
+            (unless (cdr $dependencies)
+              (setq $dependencies (copy-list '((mlist simp)))))
+            (add2lnc (cons (cons (caar z) nil) y) $dependencies))
+         (cond ((and ($subvarp (cadr zz))
+                     (not (member (caar (cadr zz)) y)))
+                (setq y (push (cadr zz) y)))
+               ((not (symbolp (cadr zz)))
+                (merror
+                  (intl:gettext "depends: argument must be a symbol; found ~M")
+                  (cadr zz)))
+               ((and (cadr zz)
+                     (not (member (cadr zz) y)))
+                (setq y (push (cadr zz) y))))))))
+  (cons '(mlist simp) (reverse res)))
 
 (defmspec $gradef (l)
   (setq l (cdr l))
   (let ((z (car l)) (n 0))
     (cond ((atom z)
-	   (if (not (= (length l) 3)) (merror "Wrong arguments to `gradef'"))
+	   (if (not (= (length l) 3)) (merror (intl:gettext "gradef: expected exactly three arguments.")))
 	   (mputprop z
 		     (cons (cons (cadr l) (meval (caddr l)))
 			   (mget z '$atomgrad))
 		     '$atomgrad)
-	   (i-$dependencies (cons (list (ncons z) (cadr l)) nil))
+	   (i-$dependencies (cons (cons (ncons z)
+	                                ;; Append existing dependencies
+	                                (cons (cadr l) (mget z 'depends)))
+	                          nil))
 	   (add2lnc z $props)
 	   z)
 	  ((or (mopp1 (caar z)) (member 'array (cdar z) :test #'eq))
-	   (merror "Wrong arguments to `gradef':~%~M" z))
+	   (merror (intl:gettext "gradef: argument cannot be a built-in operator or subscripted expression; found ~M") z))
 	  ((prog2 (setq n (- (length z) (length l))) (minusp n))
 	   (wna-err '$gradef))
 	  (t (do ((zl (cdr z) (cdr zl))) ((null zl))
 	       (if (not (symbolp (car zl)))
-		   (merror "Parameters to `gradef' must be names:~%~M"
-			   (car zl))))
+		   (merror (intl:gettext "gradef: argument must be a symbol; found ~M") (car zl))))
 	     (setq l (nconc (mapcar #'(lambda (x) (remsimp (meval x)))
 				    (cdr l))
 			    (mapcar #'(lambda (x) (list '(%derivative) z x 1))
@@ -294,6 +376,7 @@
 	     (add2lnc (cons (cons (caar z) nil) (cdr z)) $gradefs) z))))
 
 (defmfun $diff (&rest args)
+  #-gcl
   (declare (dynamic-extent args))
   (let (derivlist)
     (deriv args)))
@@ -317,7 +400,7 @@
 		((null (cddr z)) (wna-err '$diff))
 		((not (eq (ml-typep (caddr z)) 'fixnum)) (go noun))
 		((minusp (setq count (caddr z)))
-		 (merror "Improper count to `diff':~%~M" count)))
+		 (merror (intl:gettext "diff: order of derivative must be a nonnegative integer; found ~M") count)))
      loop1(cond ((zerop count) (rplacd z (cdddr z)) (go loop2))
 		((equal (setq exp (sdiff exp (cadr z))) 0) (return 0)))
      (setq count (1- count))
@@ -326,11 +409,13 @@
 
 (defun chainrule (e x)
   (let (w)
-    (cond (islinp (if (and (not (atom e))
-			   (eq (caar e) '%derivative)
-			   (not (freel (cdr e) x)))
-		      (diff%deriv (list e x 1))
-		      0))
+    (cond (*islinp*
+           ;; sdiff is called from the function islinear.
+           (if (and (not (atom e))
+                    (eq (caar e) '%derivative)
+                    (not (freel (cdr e) x)))
+               (diff%deriv (list e x 1))
+               0))
 	  ((atomgrad e x))
 	  ((not (setq w (mget (cond ((atom e) e)
 				    ((member 'array (cdar e) :test #'eq) (caar e))
@@ -355,21 +440,30 @@
   (let (y)
     (and (atom e) (setq y (mget e '$atomgrad)) (assolike x y))))
 
-(defun depends (e x)
+(defun depends (e x &aux l)
+  (setq e (specrepcheck e))
   (cond ((alike1 e x) t)
-	((mnump e) nil)
-	((atom e) (mget e 'depends))
-	(t (or (depends (caar e) x) (dependsl (cdr e) x)))))
+        ((mnump e) nil)
+        ((and (symbolp e) (setq l (mget e 'depends)))
+         ;; Go recursively through the list of dependencies.
+         ;; This code detects indirect dependencies like a(x) and x(t).
+         (dependsl l x))
+        ((atom e) nil)
+        (t (or (depends (caar e) x)
+               (dependsl (cdr e) x)))))
 
 (defun dependsl (l x)
   (dolist (u l)
     (if (depends u x) (return t))))
 
 (defmfun sdiff (e x) ; The args to SDIFF are assumed to be simplified.
+  ;; Remove a special representation from the variable of differentiation
+  (setq x (specrepcheck x))
   (cond ((alike1 e x) 1)
 	((mnump e) 0)
 	((or (atom e) (member 'array (cdar e) :test #'eq)) (chainrule e x))
 	((eq (caar e) 'mrat) (ratdx e x))
+        ((eq (caar e) 'mpois) ($poisdiff e x)) ; Poisson series
 	((eq (caar e) 'mplus) (addn (sdiffmap (cdr e) x) t))
 	((mbagp e) (cons (car e) (sdiffmap (cdr e) x)))
 	((member (caar e) '(%sum %product) :test #'eq) (diffsumprod e x))
@@ -400,8 +494,9 @@
 	((eq (caar e) '%integrate) (diffint e x))
 	((eq (caar e) '%laplace) (difflaplace e x))
 	((eq (caar e) '%at) (diff-%at e x))
-	((member (caar e) '(%realpart %imagpart) :test #'eq)
-	 (list (cons (caar e) nil) (sdiff (cadr e) x)))
+; This rule is not correct. We cut it out.
+;	((member (caar e) '(%realpart %imagpart) :test #'eq)
+;	 (list (cons (caar e) nil) (sdiff (cadr e) x)))
 	((and (eq (caar e) 'mqapply)
 	      (eq (caaadr e) '$%f))
 	 ;; Handle %f, hypergeometric function
@@ -422,31 +517,63 @@
 	(t (sdiffgrad e x))))
 
 (defun sdiffgrad (e x)
-  (let ((fun (caar e)) grad args)
+  (let ((fun (caar e)) grad args result)
     (cond ((and (eq fun 'mqapply) (oldget (caaadr e) 'grad))
-	   (sdiffgrad (cons (cons (caaadr e) nil) (append (cdadr e) (cddr e)))
-		      x))
+           ;; Change the array function f[n](x) to f(n,x), call sdiffgrad again.
+           (setq result
+	         (sdiffgrad (cons (cons (caaadr e) nil) 
+	                          (append (cdadr e) (cddr e)))
+		            x))
+           ;; If noun form for f(n,x), adjust the noun form for f[n](x)
+           (if (isinop result '%derivative)
+               (if (not (depends e x))
+                   0
+                   (diff%deriv (list e x 1)))
+               result))
 
 	  ;; extension for pdiff.
 	  ((and (get '$pderivop 'operators) (sdiffgrad-pdiff e x)))
 
+	  ;; two line extension for hypergeometric.
+	  ((and (equal fun '$hypergeometric) (get '$hypergeometric 'operators))
+	   (diff-hypergeometric (second e) (third e) (fourth e) x))
+
 	  ((or (eq fun 'mqapply) (null (setq grad (oldget fun 'grad))))
 	   (if (not (depends e x)) 0 (diff%deriv (list e x 1))))
 	  ((not (= (length (cdr e)) (length (car grad))))
-	   (merror "Wrong number of arguments for ~:M" fun))
-	  (t (setq args (sdiffmap (cdr e) x))
-	     (addn (mapcar
-		    #'mul2
-		    (cdr (substitutel
-			  (cdr e) (car grad)
-			  (do ((l1 (cdr grad) (cdr l1))
-			       (args args (cdr args)) (l2))
-			      ((null l1) (cons '(mlist) (nreverse l2)))
-			    (setq l2 (cons (cond ((equal (car args) 0) 0)
-						 (t (car l1)))
-					   l2)))))
-		    args)
-		   t)))))
+	   (merror (intl:gettext "~:M: expected exactly ~M arguments.") 
+	           fun 
+	           (length (car grad))))
+	  (t
+           (setq args (sdiffmap (cdr e) x))
+           (setq result
+                 (addn
+                   (mapcar 
+                     #'mul2
+                     (cdr 
+                       (substitutel
+                         (cdr e) 
+                         (car grad)
+                         (do ((l1 (cdr grad) (cdr l1))
+                              (args args (cdr args)) 
+                              (l2))
+                             ((null l1) (cons '(mlist) (nreverse l2)))
+                           (setq l2
+                                 (cons (cond ((equal (car args) 0) 0)
+                                             ((functionp (car l1))
+                                              ;; Evaluate a lambda expression
+                                              ;; given as a derivative.
+                                              (apply (car l1) (cdr e)))
+                                             (t (car l1)))
+                                       l2)))))
+                     args)
+                   t))
+           (if (or (null result) (not (freeof nil result)))
+               ;; A derivative has returned NIL. Return a noun form.
+               (if (not (depends e x))
+                   0 
+                   (diff%deriv (list e x 1)))
+               result)))))
 
 (defun sdiffmap (e x)
   (mapcar #'(lambda (term) (sdiff term x)) e))
@@ -518,24 +645,27 @@
 	  (mabs ((mtimes) x ((mexpt) ((mabs) x) -1)))
 	  (%erf ((mtimes) 2 ((mexpt) $%pi ((rat) -1 2))
 		 ((mexpt) $%e ((mtimes) -1 ((mexpt) x 2)))))
-	  ;;	   ($LI2 ((MTIMES) -1 ((%LOG) ((MPLUS) 1 ((MTIMES) -1 X))) ((MEXPT) X -1)))
-	  ($ei ((mtimes) ((mexpt) x -1) ((mexpt) $%e x))))))
+	  )))
 
 (defprop $atan2 ((x y) ((mtimes) y ((mexpt) ((mplus) ((mexpt) x 2) ((mexpt) y 2)) -1))
 		 ((mtimes) -1 x ((mexpt) ((mplus) ((mexpt) x 2) ((mexpt) y 2)) -1)))
   grad)
 
-(defprop $%j ((n x) ((%derivative) ((mqapply) (($%j array) n) x) n 1)
-	      ((mplus) ((mqapply) (($%j array) ((mplus) -1 n)) x)
-	       ((mtimes) -1 n ((mqapply) (($%j array) n) x) ((mexpt) x -1))))
+(defprop $li 
+  ((n x)
+; Do not put a noun form on the property list, but NIL.
+; SDIFFGRAD generates the noun form.
+;   ((%derivative) ((mqapply) (($li array) n) x) n 1)
+   nil
+   ((mtimes) ((mqapply) (($li array) ((mplus) -1 n)) x) ((mexpt) x -1)))
   grad)
 
-(defprop $li ((n x) ((%derivative) ((mqapply) (($li array) n) x) n 1)
-	      ((mtimes) ((mqapply) (($li array) ((mplus) -1 n)) x) ((mexpt) x -1)))
-  grad)
-
-(defprop $psi ((n x) ((%derivative) ((mqapply) (($psi array) n) x) n 1)
-	       ((mqapply) (($psi array) ((mplus) 1 n)) x))
+(defprop $psi 
+  ((n x)
+; Do not put a noun form on the property list, but NIL.
+; SDIFFGRAD generates the noun form.
+   nil
+   ((mqapply) (($psi array) ((mplus) 1 n)) x))
   grad)
 
 (defmfun atvarschk (argl)
@@ -553,6 +683,8 @@
 	     (if (not (mequalp u)) (return t))))))
 
 (defmfun substitutel (l1 l2 e)
+  "l1 is a list of expressions.  l2 is a list of variables. For each 
+   element in list l2, substitute corresponding element of l1 into e"
   (do ((l1 l1 (cdr l1))
        (l2 l2 (cdr l2)))
       ((null l1) e)
@@ -590,12 +722,13 @@
   (if (or (mnump e)
 	  (maxima-integerp e)
 	  (and (not (atom e)) (not (eq (caar e) 'mqapply)) (mopp1 (caar e))))
-      (merror "Non-variable 2nd argument to ~:M:~%~M" fn e)))
+      (merror (intl:gettext "~:M: second argument must be a variable; found ~M") fn e)))
 
 (defmspec $ldisplay (form)
   (disp1 (cdr form) t t))
 
 (defmfun $ldisp (&rest args)
+  #-gcl
   (declare (dynamic-extent args))
   (disp1 args t nil))
 
@@ -603,6 +736,7 @@
   (disp1 (cdr form) nil t))
 
 (defmfun $disp (&rest args)
+  #-gcl
   (declare (dynamic-extent args))
   (disp1 args nil nil))
 
@@ -662,7 +796,7 @@
 
 (defmfun $dispform (e &optional (flag nil flag?))
   (when (and flag? (not (eq flag '$all)))
-    (merror "Incorrect second argument to `dispform'"))
+    (merror (intl:gettext "dispform: second argument, if present, must be 'all'; found ~M") flag))
   (if (or (atom e)
 	  (atom (setq e (if flag? (nformat-all e) (nformat e))))
 	  (member 'simp (cdar e) :test #'eq))
@@ -683,10 +817,12 @@
       (equal ($op expr) oplist)))
 
 (defmfun $part (&rest args)
+  #-gcl
   (declare (dynamic-extent args))
   (mpart args nil nil $inflag '$part))
 
 (defmfun $inpart (&rest args)
+  #-gcl
   (declare (dynamic-extent args))
   (mpart args nil nil t '$inpart))
 
@@ -723,27 +859,30 @@
 		  (t (setq exp (nformat-all exp)))))
 	   ((specrepp exp) (setq exp (specdisrep exp))))
      (when (and (atom exp) (null $partswitch))
-       (merror "~:M called on atom: ~:M" fn exp))
+       (merror (intl:gettext "~:M: argument must be a non-atomic expression; found ~:M") fn exp))
      (when (and inflag specp)
        (setq exp (copy-tree exp)))
+     (when substflag
+       ;; Replace all ocurrences of 'rat with 'mquotient when in subst.
+       (setq exp (let (($simp nil)) (maxima-substitute 'mquotient 'rat exp))))
      (setq exp* exp)
      start (cond ((or (atom exp) (eq (caar exp) 'bigfloat)) (go err))
 		 ((equal (setq arg (if substflag (meval (car arglist)) (car arglist)))
 			 0)
 		  (setq arglist (cdr arglist))
 		  (cond ((mnump substitem)
-			 (merror "~M is an invalid operator in ~:M" substitem fn))
+			 (merror (intl:gettext "~:M: argument cannot be a number; found ~M") fn substitem))
 			((and specp arglist)
 			 (if (eq (caar exp) 'mqapply)
 			     (prog2 (setq exp (cadr exp)) (go start))
-			     (merror "Invalid operator in ~:M" fn)))
+                 ;; NOT CLEAR WHAT IS INVALID HERE. OH WELL.
+			     (merror (intl:gettext "~:M: invalid operator.") fn)))
 			(t (setq $piece (getop (mop exp)))
 			   (return
 			     (cond (substflag
 				    (setq substitem (getopr (meval substitem)))
 				    (cond ((mnump substitem)
-					   (merror "Invalid operator in ~:M:~%~M"
-						   fn substitem))
+					   (merror (intl:gettext "~:M: argument cannot be a number; found ~M") fn substitem))
 					  ((not (atom substitem))
 					   (if (not (eq (caar exp) 'mqapply))
 					       (rplaca (rplacd exp (cons (car exp)
@@ -768,7 +907,7 @@
 				      $piece))))))
 		 ((not (atom arg)) (go several))
 		 ((not (fixnump arg))
-		  (merror "Non-integer argument to ~:M:~%~M" fn arg))
+		  (merror (intl:gettext "~:M: argument must be an integer; found ~M") fn arg))
 		 ((< arg 0) (go bad)))
      (if (eq (caar exp) 'mqapply) (setq exp (cdr exp)))
      loop (cond ((not (zerop arg)) (setq arg (1- arg) exp (cdr exp))
@@ -788,9 +927,9 @@
 		((specrepp exp) (setq exp (specdisrep exp))))
      (go start)
      err  (cond ((eq $partswitch 'mapply)
-		 (merror "Improper index to list or matrix"))
+		 (merror (intl:gettext "part: invalid index of list or matrix.")))
 		($partswitch (return (setq $piece '$end)))
-		(t (merror "~:M fell off end." fn)))
+		(t (merror (intl:gettext "~:M: fell off the end.") fn)))
      bad  (improper-arg-err arg fn)
      several
      (if (or (not (member (caar arg) '(mlist $allbut) :test #'eq)) (cdr arglist))
@@ -802,7 +941,7 @@
      (setq arg1 (cdr arg) prevcount 0 exp1 exp)
      (dolist (arg* arg1)
        (if (not (fixnump arg*))
-	   (merror "Non-integer argument to ~:M:~%~M" fn arg*)))
+	   (merror (intl:gettext "~:M: argument must be an integer; found ~M") fn arg*)))
      (when (and specp (eq (caar arg) 'mlist))
        (if substflag (setq lastelem (car (last arg1))))
        (setq arg1 (sort (copy-list arg1) #'<)))
@@ -810,7 +949,7 @@
        (setq n (length exp))
        (dolist (i arg1)
 	 (if (or (< i 1) (> i n))
-	     (merror "Invalid argument to ~:M:~%~M" fn i)))
+	     (merror (intl:gettext "~:M: index must be in range 1 to ~M, inclusive; found ~M") fn n i)))
        (do ((i n (1- i)) (arg2))
 	   ((= i 0) (setq arg1 arg2))
 	 (if (not (member i arg1 :test #'equal)) (setq arg2 (cons i arg2))))
@@ -880,7 +1019,7 @@
 			    (atomchk (setq u (specrepcheck u)) '$append nil)
 			    (unless (and (alike1 op (mop u))
 					 (eq arrp (if (member 'array (cdar u) :test #'eq) t)))
-			      (merror "Arguments to `append' are not compatible."))
+			      (merror (intl:gettext "append: operators of arguments must all be the same.")))
 			    (margs u))
 			args))))))
 
@@ -895,7 +1034,7 @@
 
 (defmfun atomchk (e fun 2ndp)
   (if (or (atom e) (eq (caar e) 'bigfloat))
-      (merror "~Margument value `~M' to ~:M was not a list" (if 2ndp "2nd " "") e fun)))
+      (merror (intl:gettext "~:M: ~Margument must be a non-atomic expression; found ~M") fun (if 2ndp "2nd " "") e)))
 
 (defmfun format1 (e)
   (cond (($listp e) e)
@@ -904,7 +1043,7 @@
 
 (defmfun $first (e)
   (atomchk (setq e (format1 e)) '$first nil)
-  (if (null (cdr e)) (merror "Argument to `first' is empty."))
+  (if (null (cdr e)) (merror (intl:gettext "first: empty argument.")))
   (car (margs e)))
 
 ;; This macro is used to create functions second thru tenth.
@@ -915,7 +1054,7 @@
 	       `(defmfun ,sim (e)
 		  (atomchk (setq e (format1 e)) ',sim nil)
 		  (if (< (length (margs e)) ,i)
-		      (merror "There is no ~A element:~%~M" ',si e))
+		      (merror (intl:gettext "~:M: no such element in ~M") ',sim e))
 		  (,si (margs e))))))
 
   (make-nth second  2)
@@ -934,13 +1073,13 @@
        (return e))
      (atomchk (setq m (format1 e)) '$rest nil)
      (cond ((and n? (not (fixnump n)))
-	    (merror "2nd argument to `rest' must be an integer: ~M" n))
+	    (merror (intl:gettext "rest: second argument, if present, must be an integer; found ~M") n))
 	   ((minusp n)
 	    (setq n (- n) revp t)))
      (if (< (length (margs m)) n)
 	 (if $partswitch
 	     (return '$end)
-	     (merror "`rest' fell off end.")))
+	     (merror (intl:gettext "rest: fell off the end."))))
      (setq fun (car m))
      (when (eq (car fun) 'mqapply)
        (setq fun1 (cadr m)
@@ -957,7 +1096,7 @@
 (defmfun $last (e)
   (atomchk (setq e (format1 e)) '$last nil)
   (when (null (cdr e))
-    (merror "Argument to `last' is empty."))
+    (merror (intl:gettext "last: empty argument.")))
   (car (last e)))
 
 (defmfun $args (e)
@@ -966,7 +1105,7 @@
 
 (defmfun $delete (x l &optional (n -1 n?))
   (when (and n? (or (not (fixnump n)) (minusp n))) ; if n is set, it must be a nonneg fixnum
-    (merror "Improper 3rd argument to `delete': ~M" n))
+    (merror (intl:gettext "delete: third argument, if present, must be a nonnegative integer; found ~M") n))
   (atomchk (setq l (specrepcheck l)) '$delete t)
   (setq x (specrepcheck x)
 	l (cons (delsimp (car l)) (copy-list (cdr l))))
@@ -982,11 +1121,11 @@
   (setq e (cond (($listp e) e)
 		((or $inflag (not ($ratp e))) (specrepcheck e))
 		(t ($ratdisrep e))))
-  (cond ((symbolp e) (merror "`length' called on atomic symbol ~:M" e))
+  (cond ((symbolp e) (merror (intl:gettext "length: argument cannot be a symbol; found ~:M") e))
 	((or (numberp e) (eq (caar e) 'bigfloat))
 	 (if (and (not $inflag) (mnegp e))
 	     1
-	     (merror "`length' called on number ~:M" e)))
+	     (merror (intl:gettext "length: argument cannot be a number; found ~:M") e)))
 	((or $inflag (not (member (caar e) '(mtimes mexpt) :test #'eq))) (length (margs e)))
 	((eq (caar e) 'mexpt)
 	 (if (and (alike1 (caddr e) '((rat simp) 1 2)) $sqrtdispflag) 1 2))
@@ -1034,7 +1173,60 @@
 	 (let ((res (recur-apply #'$float e)))
 	   (if (floatp res)
 	       res
-	     (list (ncons (caar e)) ($float (cadr e)) (caddr e)))))
+	       (list (ncons (caar e)) ($float (cadr e)) (caddr e)))))
+	((and (eq (caar e) '%log)
+	      (complex-number-p (second e) '$ratnump))
+	 ;; Basically we try to compute float(log(x)) as directly as
+	 ;; possible, expecting Lisp to return some error if it can't.
+	 ;; Then we do a more complicated approach to compute the
+	 ;; result.  However, gcl and ecl don't signal errors in these
+	 ;; cases, so we always use the complicated approach for these lisps.
+	 (let ((n (second e)))
+	   (cond ((integerp n)
+		  ;; float(log(int)).  First try to compute (log
+		  ;; (float n)).  If that works, we're done.
+		  ;; Otherwise we need to do more.  
+		  (to (or #-(or gcl ecl) (ignore-errors (log (float n)))
+			  (let ((m (integer-length n)))
+			    ;; Write n as (n/2^m)*2^m where m is the number of
+			    ;; bits in n.  Then log(n) = log(2^m) + log(n/2^m).
+			    ;; n/2^m is approximately 1, so converting that to a
+			    ;; float is no problem.  log(2^m) = m * log(2).
+			    (+ (* m (log 2e0))
+			       (log (float (/ n (ash 1 m)))))))))
+		 (($ratnump n)
+		  ;; float(log(n/m)) where n and m are integers.  Try computing
+		  ;; it first.  If it fails, compute as log(n) - log(m).
+		  (let ((try #-(or gcl ecl)
+			     (ignore-errors (log (fpcofrat n)))))
+		    (if try
+			(to try)
+			(sub  ($float `((%log) ,(second n)))
+			      ($float `((%log) ,(third n)))))))
+		 ((complex-number-p n 'integerp)
+		  ;; float(log(n+m*%i)).
+		  (let ((re ($realpart n))
+			(im ($imagpart n)))
+		    (to (or #-(or gcl ecl)
+			    (ignore-errors (log (complex (float re)
+							 (float im))))
+			    (let* ((size (max (integer-length re)
+					      (integer-length im)))
+				   (scale (ash 1 size)))
+			      (+ (* size (log 2e0))
+				 (log (complex (float (/ re scale))
+					       (float (/ im scale))))))))))
+		 (t
+		  ;; log(n1/d1 + n2/d2*%i) =
+		  ;;   log(s*(n+m*%i)) = log(s) + log(n+m*%i)
+		  ;;
+		  ;; where s = lcm(d1, d2), n and m are integers
+		  ;;
+		  (let* ((s (lcm ($denom ($realpart n))
+				 ($denom ($imagpart n))))
+			 (p ($expand (mul s n))))
+		    (add ($float `((%log) ,s))
+			 ($float `((%log) ,p))))))))
 	(t (recur-apply #'$float e))))
 
 (defmfun $coeff (e x &optional (n 1))
@@ -1042,7 +1234,7 @@
       (coeff e x 0)
       (coeff e (power x n) 1)))
 
-(defmfun coeff (e var pow)
+(defun coeff (e var pow)
   (simplify
    (cond ((alike1 e var) (if (equal pow 1) 1 0))
 	 ((atom e) (if (equal pow 0) e 0))
@@ -1055,12 +1247,22 @@
 	  (cons (if (eq (caar e) 'mplus) '(mplus) (car e))
 		(mapcar #'(lambda (e) (coeff e var pow)) (cdr e))))
 	 ((eq (caar e) 'mrat) (ratcoeff e var pow))
-	 ((equal pow 0) (if (free e var) e 0))
+         ((equal pow 0) (if (coeff-contains-powers e var) 0 e))
 	 ((eq (caar e) 'mtimes)
 	  (let ((term (if (equal pow 1) var (power var pow))))
 	    (if (memalike term (cdr e)) ($delete term e 1) 0)))
 	 (t 0))))
 
+(defun coeff-contains-powers (e var)
+  (cond ((alike1 e var) t)
+        ((atom e) nil)
+        ((eq (caar e) 'mexpt)
+         (alike1 (cadr e) var))
+        ((eq (caar e) 'mtimes)
+         (member t (mapcar #'(lambda (e)
+                               (coeff-contains-powers e var)) (cdr e))))
+        (t nil)))
+
 (declare-top (special powers var hiflg num flag))
 
 (defmfun $hipow (e var)
