@@ -16,8 +16,6 @@
 
 (defun chrct* () (- linel chrps))
 
-(defvar fortranp nil)
-
 (defmspec $grind (x)
   (setq x (cdr x))
   (let (y)
@@ -131,10 +129,10 @@
     (t (setq x  (cdr x))
        (do () ((null x) l) (setq l (string1 (car x) l) x (cdr x))))))
 
-
 (defun msize (x l r lop rop)
   (setq x (nformat x))
-  (cond ((atom x) (if fortranp (msz (makestring x) l r) (msize-atom x l r)))
+  (cond ((atom x) (msize-atom x l r))
+        ((and (atom (car x)) (setq x (cons '(mprogn) x)) nil))
 	((or (<= (lbp (caar x)) (rbp lop)) (> (lbp rop) (rbp (caar x))))
 	 (msize-paren x l r))
 	((member 'array (cdar x) :test #'eq) (msize-array x l r))
@@ -145,26 +143,29 @@
 (defun msize-atom (x l r)
   (prog (y)
      (cond ((numberp x) (setq y (exploden x)))
-	   ((and (setq y (safe-get x 'reversealias))
-		 (not (and (member x $aliases :test #'eq) (get x 'noun))))
-	    (setq y (exploden (stripdollar y))))
-	   ((setq y (rassoc x aliaslist :test #'eq)) (return (msize (car y) l r lop rop)))
-       ((null (setq y (exploden x))))
-       ((safe-get x 'noun) (return (msize-atom (get x 'noun) l r)))
-	   ((char= #\$ (car y)) (setq y (slash (cdr y))))
-	   ((stringp x)
-        (setq y (coerce x 'list))
-	    (do ((l y (cdr l))) ((null l))
-	      (cond ((or (member (car l) '(#\" #\\ #\; #\$) :test #'equal)
-			 (and (char< (car l) #\space)
-			      (not (char= (car l) #\return))))
-		     (rplacd l (cons (car l) (cdr l)))
-		     (rplaca l #\\) (setq l (cdr l)))))
-	    (setq y (cons #\" (nconc y (list #\")))))
-	   (t (setq y (cons #\? (slash y)))))
+           ((stringp x)
+            (setq y (coerce x 'list))
+            (do ((l y (cdr l))) ((null l))
+              (cond ((member (car l) '(#\" #\\ ) :test #'equal)
+                     (rplacd l (cons (car l) (cdr l)))
+                     (rplaca l #\\ )
+                     (setq l (cdr l)))))
+            (setq y (cons #\" (nconc y (list #\")))))
+           ((and (setq y (safe-get x 'reversealias))
+                 (not (and (member x $aliases :test #'eq) (get x 'noun))))
+            (setq y (exploden (stripdollar y))))
+           ((setq y (rassoc x aliaslist :test #'eq))
+            (return (msize (car y) l r lop rop)))
+           ((null (setq y (exploden x))))
+           ((safe-get x 'noun) (return (msize-atom (get x 'noun) l r)))
+           ((char= #\$ (car y)) (setq y (slash (cdr y))))
+           ((member (marray-type x) '(array hash-table $functional))
+            (return (msize-array-object x l r)))
+           (t (setq y (cons #\? (slash y)))))
      (return (msz y l r))))
 
-(defun msz (x l r) (setq x (nreconc l (nconc x r))) (cons (length x) x))
+(defun msz (x l r)
+  (setq x (nreconc l (nconc x r))) (cons (length x) x))
 
 (defun slash (x)
   (do ((l (cdr x) (cdr l))) ((null l))
@@ -268,8 +269,47 @@
   (msz (mapcar #'(lambda (l) (getcharn l 1)) (fpformat x)) l r))
 
 (defprop mprogn msize-matchfix grind)
+(defprop mprogn ((#\( ) #\) ) strsym)
+
 (defprop mlist msize-matchfix grind)
 
+;;; ----------------------------------------------------------------------------
+
+;; Formating a mlable-expression
+
+(defprop mlable msize-mlable grind)
+
+(defun msize-mlable (x l r)
+  (declare (special *display-labels-p*))
+  (if *display-labels-p*
+      (setq l (cons (msize (cadr x) (list #\( ) (list #\) #\ ) nil nil) l)))
+  (msize (caddr x) l r lop rop))
+
+;;; ----------------------------------------------------------------------------
+
+;; Formating a mtext-expression
+
+(defprop mtext msize-mtext grind)
+
+(defun msize-mtext (x l r)
+  (setq x (cdr x))
+  (if (null x)
+      (msz nil l r)
+      (do ((nl) (w 0))
+          ((null (cdr x))
+           (setq nl (cons (if (atom (car x))
+                              (msz (makestring (car x)) l r)
+                              (msize (car x) l r lop rop))
+                          nl))
+           (cons (+ w (caar nl)) (nreverse nl)))
+        (setq nl (cons (if (atom (car x))
+                           (msz (makestring (car x)) l r)
+                           (msize (car x) l r lop rop))
+                       nl)
+              w (+ w (caar nl))
+              x (cdr x)
+              l nil))))
+
 (defprop mqapply msz-mqapply grind)
 
 (defun msz-mqapply (x l r)
@@ -302,26 +342,42 @@
 (defprop mset 180. lbp)
 (defprop mset 20. rbp)
 
+;;; ----------------------------------------------------------------------------
+
+;; Formating a mdefine or mdefmacro expression
+
 (defprop mdefine msz-mdef grind)
 (defprop mdefine (#\: #\=) strsym)
-(defprop mdefine 180. lbp)
-(defprop mdefine 20. rbp)
+(defprop mdefine 180 lbp)
+(defprop mdefine  20 rbp)
 
 (defprop mdefmacro msz-mdef grind)
 (defprop mdefmacro (#\: #\: #\=) strsym)
-(defprop mdefmacro 180. lbp)
-(defprop mdefmacro 20. rbp)
+(defprop mdefmacro 180 lbp)
+(defprop mdefmacro  20 rbp)
 
 (defun msz-mdef (x l r)
   (setq l (msize (cadr x) l (copy-list (strsym (caar x))) lop (caar x))
-	r (msize (caddr x) nil r (caar x) rop))
-  (setq x (cons (- (car l) (caadr l)) (cddr l)))
-  (if (and (not (atom (cadr r))) (not (atom (caddr r)))
-	   (< (+ (car l) (caadr r) (caaddr r)) linel))
-      (setq x (nconc x (list (cadr r) (caddr r)))
-	    r (cons (car r) (cdddr r))))
-  (cons (+ (car l) (car r)) (cons (cadr l) (cons x (cdr r)))))
-
+        r (msize (caddr x) nil r (caar x) rop))
+  (cond ((not (atom (cadr l)))
+         ;; An expression like g(x):=x:
+         ;;   left side  l = (6 (2 #\g #\( ) (4 #\x #\) #\: #\= ))
+         ;;   right side r = (1 #\x )
+         ;; the result is (7 (2 #\g #\( ) (4 #\x #\) #\: #\= ) (1 #\x ))
+         (setq x (cons (- (car l) (caadr l)) (cddr l)))
+         (if (and (not (atom (cadr r)))
+                  (not (atom (caddr r)))
+                  (< (+ (car l) (caadr r) (caaddr r)) linel))
+             (setq x (nconc x (list (cadr r) (caddr r)))
+                   r (cons (car r) (cdddr r))))
+         (cons (+ (car l) (car r)) (cons (cadr l) (cons x (cdr r)))))
+        (t
+         ;; An expression like x f :=x or f x:=x, where f is a postfix or a
+         ;; prefix operator. Example for a postfix operator:
+         ;;   left side  l = (5 #\x #\space #\f #\: #\= )
+         ;;   right side r = (1 #\x)
+         ;; the result is (6 (5 #\x #\space #\f #\: #\=) (1 #\x))
+         (cons (+ (car l) (car r)) (cons l (ncons r))))))
 
 (defprop mfactorial msize-postfix grind)
 (defprop mfactorial 160. lbp)
@@ -383,11 +439,36 @@
 		   w (+ (caar nl) w)
 		   x (cdr x))))))
 
-(defprop mminus msize-prefix grind)
+(defprop mminus msize-mminus grind)
 (defprop mminus (#\-) strsym)
 (defprop mminus 100. rbp)
 (defprop mminus 100. lbp)
 
+(defun msize-mminus (x l r)
+  (cond ((null (cddr x))
+         (if (null (cdr x))
+             (msize-function x l r t)
+             (msize (cadr x) (append (ncons #\- ) l) r 'mminus rop)))
+        (t
+         (setq l (msize (cadr x) l nil lop 'mminus)
+               x (cddr x))
+         (do ((nl (list l))
+              (w (car l))
+              (dissym))
+             ((null (cdr x))
+              (if (mmminusp (car x))
+                  (setq l (cadar x) dissym (list #\+ ))
+                  (setq l (car x) dissym (list #\- )))
+              (setq r (msize l dissym r 'mminus rop))
+              (cons (+ (car r) w) (nreverse (cons r nl))))
+           (declare (fixnum w))
+           (if (mmminusp (car x))
+               (setq l (cadar x) dissym (list #\+ ))
+               (setq l (car x) dissym (list #\- )))
+           (setq nl (cons (msize l dissym nil 'mminus 'mminus) nl)
+                 w (+ (caar nl) w)
+                 x (cdr x))))))
+
 (defprop mequal msize-infix grind)
 (defprop mequal 80. lbp)
 (defprop mequal 80. rbp)
@@ -523,7 +604,7 @@
 	       ((fifth x)  `($next ,(fifth x))))
 	 (cond ((sixth x)  `($thru ,(sixth x))))
 	 (cond ((null (seventh x)) nil)
-	       ((eq 'mnot (caar (seventh x)))
+	       ((and (consp (seventh x)) (eq 'mnot (caar (seventh x))))
 		`($while ,(cadr (seventh x))))
 	       (t `($unless ,(seventh x))))
 	 `($do ,(eighth x))))
@@ -532,7 +613,7 @@
   (nconc `($for ,(second x) $in ,(third x))
 	 (cond ((sixth x) `($thru ,(sixth x))))
 	 (cond ((null (seventh x)) nil)
-	       ((eq 'mnot (caar (seventh x)))
+	       ((and (consp (seventh x)) (eq 'mnot (caar (seventh x))))
 		`($while ,(cadr (seventh x))))
 	       (t `($unless ,(seventh x))))
 	 `($do ,(eighth x))))
