@@ -122,7 +122,7 @@
 	(cl-info::*prompt-suffix* *prompt-suffix*))
     (if exact-p
 	(cl-info::info-exact topic)
-	(cl-info::info topic))))
+	(cl-info::info-inexact topic))))
 
 ; The old implementation
 ;(defun $apropos (s)
