@@ -5,10 +5,14 @@
 ;;; support double-float, (complex double-float) and Maxima bfloat and
 ;;; complex bfloat arithmetic, without having to write separate
 ;;; versions for each.  Of course, specially written versions for
-;;; double-float and (cmoplex double-float) will probably be much
+;;; double-float and (complex double-float) will probably be much
 ;;; faster, but this allows users to write just one routine that can
 ;;; handle all of the data types in a more "natural" Lisp style.
 
+#+cmu
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (setf lisp::*enable-package-locked-errors* nil))
+
 (in-package #-gcl #:bigfloat #+gcl "BIGFLOAT")
 
 (defun intofp (re)
@@ -129,7 +133,7 @@
 
 (defmethod maxima::to ((z cl:rational))
   (if (typep z 'ratio)
-      (maxima::div (numerator z) (denominator z))
+      (list '(maxima::rat maxima::simp) (numerator z) (denominator z))
       z))
 
 (defmethod maxima::to ((z cl:complex))
@@ -146,7 +150,10 @@
   (maxima::add (real-value z)
 	       (maxima::mul 'maxima::$%i
 			    (imag-value z))))
-  
+
+(defmethod maxima::to ((z t))
+  z)
+
 ;;; REALP
 
 ;; GCL doesn't have the REAL class!  But since a real is a rational or
@@ -1448,7 +1455,25 @@
 	  (bigfloat 1))
       (if (and (zerop a) (plusp (realpart b)))
 	  (* a b)
-	  (exp (* (bigfloat b) (log a))))))
+	  ;; Handle a few special cases using multiplication.
+	  (cond ((= b 1)
+		 a)
+		((= b -1)
+		 (/ a))
+		((= b 2)
+		 (* a a))
+		((= b -2)
+		 (/ (* a a)))
+		((= b 3) (* a a a))
+		((= b -3) (/ (* a a a)))
+		((= b 4)
+		 (let ((a2 (* a a)))
+		   (* a2 a2)))
+		((= b -4)
+		 (let ((a2 (* a a)))
+		   (/ (* a2 a2))))
+		(t
+		 (exp (* (bigfloat b) (log a))))))))
 
 ;; Handle a^b a little more carefully because the result is known to
 ;; be real when a is real and b is an integer.
@@ -1458,8 +1483,19 @@
 	((and (zerop a) (plusp b))
 	 ;; 0^b, for positive b
 	 (* a b))
+	;; Handle a few special cases using multiplication.
 	((eql b 1) a)
 	((eql b -1) (/ a))
+	((eql b 2) (* a a))
+	((eql b -2) (/ (* a a)))
+	((eql b 3) (* a a a))
+	((eql b -3) (/ (* a a a)))
+	((eql b 4)
+	 (let ((a2 (* a a)))
+	   (* a2 a2)))
+	((eql b -4)
+	 (let ((a2 (* a a)))
+	   (/ (* a2 a2))))
 	((minusp a)
 	 ;; a^b = exp(b*log(|a|) + %i*%pi*b)
 	 ;;     = exp(b*log(|a|))*exp(%i*%pi*b)
@@ -1476,8 +1512,41 @@
 ;;; bigfloats are convert to BIGFLOATS.  Maxima complex numbers are
 ;;; converted to CL complex numbers or to COMPLEX-BIGFLOAT's.
 (defun to (maxima-num &optional imag)
+  (let ((result (ignore-errors (%to maxima-num imag))))
+    (or result
+	(maxima::merror (intl:gettext "BIGFLOAT: unable to convert ~M to a CL or BIGFLOAT number.")
+			(if imag
+			    (maxima::add maxima-num (maxima::mul imag 'maxima::$%i))
+			    maxima-num)))))
+
+;;; MAYBE-TO - External
+;;;
+;;;   Like TO, but if the maxima number can't be converted to a CL
+;;; number or BIGFLOAT, just return the maxima number.
+(defun maybe-to (maxima-num &optional imag)
+  (let ((result (ignore-errors (%to maxima-num imag))))
+    (or result
+	(if imag
+	    (maxima::add maxima-num imag)
+	    maxima-num))))
+	
+(defun %to (maxima-num &optional imag)
   (cond (imag
-	 (complex (to maxima-num) (to imag)))
+	 ;; Clisp has a "feature" that (complex rat float) does not
+	 ;; make the both components of the complex number a float.
+	 ;; Sometimes this is nice, but other times it's annoying
+	 ;; because it is non-ANSI behavior.  For our code, we really
+	 ;; want both components to be a float.
+	 #-clisp
+	 (complex (to maxima-num) (to imag))
+	 #+clisp
+	 (let ((re (to maxima-num))
+	       (im (to imag)))
+	   (cond ((and (rationalp re) (floatp im))
+		  (setf re (float re im)))
+		 ((and (rational im) (floatp re))
+		  (setf im (float im re))))
+	   (complex re im)))
 	(t
 	 (cond ((cl:realp maxima-num)
 		maxima-num)
@@ -1506,7 +1575,7 @@
 		    (typep maxima-num 'complex-bigfloat))
 		maxima-num)
 	       (t
-		(maxima::merror (intl:gettext "BIGFLOAT: unable to convert ~M to a CL or BIGFLOAT number.") maxima-num))))))
+		(error "BIGFLOAT: unable to convert to a CL or BIGFLOAT number."))))))
 
 ;;; EPSILON - External
 ;;;
@@ -1667,7 +1736,7 @@
 
 
 (defmethod float ((x bigfloat) (y cl:float))
-  (if (typep y 'double-float)
+  (if (typep y 'maxima::flonum)
       (maxima::fp2flo (real-value x))
       (fp2single (real-value x))))
 
@@ -1872,6 +1941,9 @@
 	   ;; (coerce bigfloat foo)
 	   (cond ((subtypep type 'cl:float)
 		  (float obj (cl:coerce 0 type)))
+		 ((subtypep type '(cl:complex long-float))
+		  (cl:complex (float (realpart obj) 1l0)
+			      (float (imagpart obj) 1l0)))
 		 ((subtypep type '(cl:complex double-float))
 		  (cl:complex (float (realpart obj) 1d0)
 			      (float (imagpart obj) 1d0)))
@@ -1882,15 +1954,18 @@
 		  ;; What should we do here?  Return a
 		  ;; complex-bigfloat?  A complex double-float?
 		  ;; complex single-float?  I arbitrarily select
-		  ;; complex double-float for now.
-		  (cl:complex (float (realpart obj) 1d0)
-			      (float (imagpart obj) 1d0)))
+		  ;; complex maxima:flonum for now.
+		  (cl:complex (float (realpart obj) 1.0)
+			      (float (imagpart obj) 1.0)))
 		 (t
 		  (coerce-error))))
 	  ((typep obj 'complex-bigfloat)
 	   ;; (coerce complex-bigfloat foo)
 	   (cond ((subtypep type 'complex-bigfloat)
 		  obj)
+		 ((subtypep type '(cl:complex long-float))
+		  (cl:complex (float (realpart obj) 1l0)
+			      (float (imagpart obj) 1l0)))
 		 ((subtypep type '(cl:complex double-float))
 		  (cl:complex (float (realpart obj) 1d0)
 			      (float (imagpart obj) 1d0)))
