@@ -19,15 +19,15 @@
 (macsyma-module outmis)
 
 
-(declare-top (special *xvar $exptisolate $labels $dispflag errorsw)) 
+(declare-top (special *xvar $exptisolate $labels $dispflag errorsw))
 
 (defmvar $exptisolate nil)
 (defmvar $isolate_wrt_times nil)
 
 (defmfun $isolate (e *xvar)
-  (setq *xvar (getopr *xvar)) (iso1 e)) 
+  (setq *xvar (getopr *xvar)) (iso1 e))
 
-(defun iso1 (e) 
+(defun iso1 (e)
   (cond ((specrepp e) (iso1 (specdisrep e)))
 	((and (free e 'mplus) (or (null $isolate_wrt_times) (free e 'mtimes))) e)
 	((freeof *xvar e) (mgen2 e))
@@ -48,8 +48,8 @@
 			  (t u))))))
 	(t (cons (car e) (mapcar #'iso1 (cdr e))))))
 
-(defun iso2 (e) 
-  (prog (hasit doesnt op) 
+(defun iso2 (e)
+  (prog (hasit doesnt op)
      (setq op (ncons (caar e)))
      (do ((i (cdr e) (cdr i))) ((null i))
        (cond ((freeof *xvar (car i)) (setq doesnt (cons (car i) doesnt)))
@@ -62,44 +62,44 @@
 			 (free doesnt 'mtimes)))))
 	   (t (setq doesnt (mgen2 doesnt))))
      (setq doesnt (ncons doesnt))
-     ret  (return (simplifya (cons op (nconc hasit doesnt)) nil)))) 
+     ret  (return (simplifya (cons op (nconc hasit doesnt)) nil))))
 
 (defun mgen2 (h)
   (cond ((memsimilarl h (cdr $labels) (getlabcharn $linechar)))
-	(t (setq h (displine h)) (and $dispflag (mterpri)) h))) 
+	(t (setq h (displine h)) (and $dispflag (mterpri)) h)))
 
-(defun memsimilarl (item list linechar) 
+(defun memsimilarl (item list linechar)
   (cond ((null list) nil)
 	((and (char= (getlabcharn (car list)) linechar)
 	      (boundp (car list))
 	      (memsimilar item (car list) (symbol-value (car list)))))
-	(t (memsimilarl item (cdr list) linechar)))) 
+	(t (memsimilarl item (cdr list) linechar))))
 
-(defun memsimilar (item1 item2 item2ev) 
+(defun memsimilar (item1 item2 item2ev)
   (cond ((equal item2ev 0) nil)
 	((alike1 item1 item2ev) item2)
 	(t (let ((errorsw t) r)
 	     (setq r (catch 'errorsw (div item2ev item1)))
-	     (and (mnump r) (not (zerop r)) (div item2 r))))))
+	     (and (mnump r) (not (zerop1 r)) (div item2 r))))))
 
 (defmfun $pickapart (x lev)
   (setq x (format1 x))
   (cond ((not (fixnump lev))
-	 (merror "Improper 2nd argument to `pickapart':~%~M" lev))
+	 (merror (intl:gettext "pickapart: second argument must be an integer; found: ~M") lev))
 	((or (atom x) (and (eq (caar x) 'mminus) (atom (cadr x)))) x)
 	((= lev 0) (mgen2 x))
 	((and (atom (cdr x)) (cdr x)) x)
-	(t (cons (car x) (mapcar #'(lambda (y) ($pickapart y (1- lev))) (cdr x)))))) 
+	(t (cons (car x) (mapcar #'(lambda (y) ($pickapart y (1- lev))) (cdr x))))))
 
-(defmfun $reveal (e lev) 
+(defmfun $reveal (e lev)
   (setq e (format1 e))
   (cond ((and (eq (ml-typep lev) 'fixnum) (> lev 0)) (reveal e 1 lev))
-	(t (merror "Second argument to reveal must be positive integer."))))
+	(t (merror (intl:gettext "reveal: second argument must be a positive integer; found: ~M") lev))))
 
 (defun simple (x)
-  (or (atom x) (member (caar x) '(rat bigfloat) :test #'eq))) 
+  (or (atom x) (member (caar x) '(rat bigfloat) :test #'eq)))
 
-(defun reveal (e nn lev) 
+(defun reveal (e nn lev)
   (cond ((simple e) e)
 	((= nn lev)
 	 (cond ((eq (caar e) 'mplus) (cons '(|$Sum| simp) (ncons (length (cdr e)))))
@@ -107,6 +107,8 @@
 	       ((eq (caar e) 'mexpt) '|$Expt|)
 	       ((eq (caar e) 'mquotient) '|$Quotient|)
 	       ((eq (caar e) 'mminus) '|$Negterm|)
+	       ((eq (caar e) 'mlist)
+	        (cons '(|$List| simp) (ncons (length (cdr e)))))
 	       (t (getop (mop e)))))
 	(t (let ((u (cond ((member 'simp (cdar e) :test #'eq) (car e))
 			  (t (cons (caar e) (cons 'simp (cdar e))))))
@@ -117,12 +119,13 @@
 		   (t (cons u v)))))))
 
 (declare-top (special atvars munbound $props $gradefs $features opers
-		      $contexts $activecontexts $aliases)) 
+		      $contexts $activecontexts $aliases))
 
 (defmspec $properties (x)
   (setq x (getopr (fexprcheck x)))
   (unless (or (symbolp x) (stringp x))
-    (merror "properties: argument must be a symbol or a string."))
+    (merror
+      (intl:gettext "properties: argument must be a symbol or a string.")))
   (let ((u (properties x)) (v (or (safe-get x 'noun) (safe-get x 'verb))))
     (if v (nconc u (cdr (properties v))) u)))
 
@@ -133,47 +136,62 @@
       '((mlist) $alphabetic)
       '((mlist)))
     (do ((y (symbol-plist x) (cddr y))
-         (l (cons '(mlist simp) (and (boundp x)
+	 (l (cons '(mlist simp) (and (boundp x)
   				   (if (optionp x) (ncons "system value")
   				       (ncons '$value)))))
-         (prop))
-        ((null y)
-         
-         (if (member x (cdr $features) :test #'eq) (nconc l (ncons '$feature)))
-         (if (member x (cdr $contexts) :test #'eq) (nconc l (ncons '$context)))
-         (if (member x (cdr $activecontexts) :test #'eq)
+	 (prop))
+	((null y)
+	 (if (member x (cdr $features) :test #'eq) (nconc l (ncons '$feature)))
+	 (if (member x (cdr $contexts) :test #'eq) (nconc l (ncons '$context)))
+	 (if (member x (cdr $activecontexts) :test #'eq)
   	   (nconc l (ncons '$activecontext)))
-         (cond  ((null (symbol-plist x))
+	 (cond  ((null (symbol-plist x))
   	       (if (fboundp x) (nconc l (list "system function")))))
-  
-         l)
-      
-      ;; TOP-LEVEL PROPERTIES 
-      (cond ((setq prop (assoc (car y)
-  			    `((bindtest . $bindtest)
-  			      (sp2 . $deftaylor) (sp2subs . $deftaylor)
-  			      (assign . "assign property")
-  			      (nonarray . $nonarray) (grad . $gradef)
-  			      (noun . $noun) (evfun . $evfun) (special . $special)
-  			      (evflag . $evflag) (op . $operator)) :test #'eq))
+	 l)
+
+      ;; TOP-LEVEL PROPERTIES
+      (cond ((setq prop 
+                   (assoc (car y)
+                          `((bindtest . $bindtest)
+                            (sp2 . $deftaylor)
+                            (sp2subs . $deftaylor)
+                            (assign . "assign property")
+                            (nonarray . $nonarray)
+                            (grad . $gradef)
+                            (integral . $integral)
+                            (distribute_over . "distributes over bags")
+                            (simplim%function . "limit function")
+                            (conjugate-function . "conjugate function")
+                            (commutes-with-conjugate . "mirror symmetry")
+  			    (risplit-function . "complex characteristic")
+                            (noun . $noun)
+                            (evfun . $evfun)
+                            (evflag . $evflag)
+                            (op . $operator)) :test #'eq))
   	   (nconc l (ncons (cdr prop))))
-  	  ((setq prop (member (car y) opers :test #'eq)) (nconc l (list (car prop))))
+  	  ((setq prop (member (car y) opers :test #'eq)) 
+  	   (nconc l (list (car prop))))
   	  ((and (eq (car y) 'operators) (not (eq (cadr y) 'simpargs1)))
   	   (nconc l (list '$rule)))
   	  ((and (member (car y) '(fexpr fsubr mfexpr*s mfexpr*) :test #'eq)
   		(nconc l (ncons "special evaluation form"))
   		nil))
   	  ((and (or (get (car y) 'mfexpr*) (fboundp x))
+  	        ;; Do not add more than one entry to the list.
+  	        (not (member '$transfun l))
+  	        (not (member '$rule l))
   		(not (member "system function" l :test #'equal)))
   	   (nconc l
   		  (list (cond ((get x 'translated) '$transfun)
   			      ((mgetl x '($rule ruleof)) '$rule)
   			      (t "system function")))))
-  	  ((and (eq (car y) 'autoload) (not (member "system function" l :test #'equal)))
+  	  ((and (eq (car y) 'autoload) 
+  	        (not (member "system function" l :test #'equal)))
   	   (nconc l (ncons (if (member x (cdr $props) :test #'eq)
   			       "user autoload function"
   			       "system function"))))
-  	  ((and (eq (car y) 'reversealias) (member (car y) (cdr $aliases) :test #'eq))
+  	  ((and (eq (car y) 'reversealias) 
+  	        (member (car y) (cdr $aliases) :test #'eq))
   	   (nconc l (ncons '$alias)))
   	  ((eq (car y) 'data)
   	   (nconc l (cons "database info" (cdr ($facts x)))))
@@ -192,7 +210,6 @@
   				       ($atomgrad . $atomgrad)
   				       ($numer . $numer)
   				       (depends . $dependency)
-  				       ($constant . $constant)
   				       ($nonscalar . $nonscalar)
   				       ($scalar . $scalar)
   				       (matchdeclare . $matchdeclare)
@@ -209,7 +226,6 @@
   			       ((null y) (list l))
   			     (nconc l (list (car y)))))))))))))
 
-
 (defmspec $propvars (x)
   (setq x (fexprcheck x))
   (do ((iteml (cdr $props) (cdr iteml)) (propvars (ncons '(mlist))))
@@ -218,7 +234,7 @@
 	 (nconc propvars (ncons (car iteml))))))
 
 (defmspec $printprops (r) (setq r (cdr r))
-	  (if (null (cdr r)) (merror "`printprops' takes two arguments."))
+	  (if (null (cdr r)) (merror (intl:gettext "printprops: requires two arguments.")))
 	  (let ((s (cadr r)))
 	    (setq r (car r))
 	    (setq r (cond ((atom r)
@@ -231,48 +247,47 @@
 		  ((eq s '$atomgrad) (dispatomgrads r))
 		  ((eq s '$gradef) (dispgradefs r))
 		  ((eq s '$matchdeclare) (dispmatchdeclares r))
-		  (t (merror "Unknown `property' - `printprops':  ~:M" s)))))
+		  (t (merror (intl:gettext "printprops: unknown property ~:M") s)))))
 
-(defun dispatvalues (l) 
+(defun dispatvalues (l)
   (do ((l l (cdr l)))
       ((null l))
     (do ((ll (mget (car l) 'atvalues) (cdr ll)))
 	((null ll))
       (mtell-open "~M~%"
-		  (list '(mlable) nil 
+		  (list '(mlable) nil
 			(list '(mequal)
 			      (atdecode (car l) (caar ll) (cadar ll)) (caddar ll))))))
   '$done)
 
 ;;(declare-top (FIXNUM N))
 
-(defun atdecode (fun dl vl) 
+(defun atdecode (fun dl vl)
   (setq vl (copy-list vl))
   (atvarschk vl)
-  ((lambda (eqs nvarl)
-     (cond ((not (member nil (mapcar #'(lambda (x) (signp e x)) dl) :test #'eq))
-	    (do ((vl vl (cdr vl)) (varl atvars (cdr varl)))
-		((null vl))
-	      (and (eq (car vl) munbound) (rplaca vl (car varl))))
-	    (cons (list fun) vl))
-	   (t (setq fun (cons (list fun)
-			      (do ((n (length vl) (1- n))
-				   (varl atvars (cdr varl))
-				   (l nil (cons (car varl) l)))
-				  ((zerop n) (nreverse l)))))
-	      (do ((vl vl (cdr vl)) (varl atvars (cdr varl)))
-		  ((null vl))
-		(and (not (eq (car vl) munbound))
-		     (setq eqs (cons (list '(mequal) (car varl) (car vl)) eqs))))
-	      (setq eqs (cons '(mlist) (nreverse eqs)))
-	      (do ((varl atvars (cdr varl)) (dl dl (cdr dl)))
-		  ((null dl) (setq nvarl (nreverse nvarl)))
-		(and (not (zerop (car dl)))
-		     (setq nvarl (cons (car dl) (cons (car varl) nvarl)))))
-	      (list '(%at) (cons '(%derivative) (cons fun nvarl)) eqs))))
-   nil nil)) 
+  (let ((eqs nil) (nvarl nil))
+    (cond ((not (member nil (mapcar #'(lambda (x) (signp e x)) dl) :test #'eq))
+	   (do ((vl vl (cdr vl)) (varl atvars (cdr varl)))
+	       ((null vl))
+	     (and (eq (car vl) munbound) (rplaca vl (car varl))))
+	   (cons (list fun) vl))
+	  (t (setq fun (cons (list fun)
+			     (do ((n (length vl) (1- n))
+				  (varl atvars (cdr varl))
+				  (l nil (cons (car varl) l)))
+				 ((zerop n) (nreverse l)))))
+	     (do ((vl vl (cdr vl)) (varl atvars (cdr varl)))
+		 ((null vl))
+	       (and (not (eq (car vl) munbound))
+		    (setq eqs (cons (list '(mequal) (car varl) (car vl)) eqs))))
+	     (setq eqs (cons '(mlist) (nreverse eqs)))
+	     (do ((varl atvars (cdr varl)) (dl dl (cdr dl)))
+		 ((null dl) (setq nvarl (nreverse nvarl)))
+	       (and (not (zerop (car dl)))
+		    (setq nvarl (cons (car dl) (cons (car varl) nvarl)))))
+	     (list '(%at) (cons '(%derivative) (cons fun nvarl)) eqs)))))
 
-(defun dispatomgrads (l) 
+(defun dispatomgrads (l)
   (do ((i l (cdr i)))
       ((null i))
     (do ((j (mget (car i) '$atomgrad) (cdr j)))
@@ -281,9 +296,9 @@
 		  (list '(mlable) nil
 			(list '(mequal)
 			      (list '(%derivative) (car i) (caar j) 1) (cdar j))))))
-  '$done) 
+  '$done)
 
-(defun dispgradefs (l) 
+(defun dispgradefs (l)
   (do ((i l (cdr i)))
       ((null i))
     (setq l (get (car i) 'grad))
@@ -294,9 +309,9 @@
       (mtell-open "~M~%"
 		  (list '(mlable)
 			nil (list '(mequal) (list '(%derivative) thing (car j) 1.) (car k))))))
-  '$done) 
+  '$done)
 
-(defun dispmatchdeclares (l) 
+(defun dispmatchdeclares (l)
   (do ((i l (cdr i))
        (ret))
       ((null i) (cons '(mlist) ret))
@@ -308,12 +323,12 @@
 (declare-top (special trans ovar nvar tfun invfun $programmode nfun
 		      *roots *failures varlist genvar $ratfac))
 
-(defmfun $changevar (expr trans nvar ovar) 
+(defmfun $changevar (expr trans nvar ovar)
   (let (invfun nfun $ratfac)
     (cond ((or (atom expr) (eq (caar expr) 'rat) (eq (caar expr) 'mrat))  expr)
-	  ((atom trans) (merror "2nd arg must not be atomic"))
-	  ((null (atom nvar)) (merror "3rd arg must be atomic"))
-	  ((null (atom ovar)) (merror "4th arg must be atomic")))
+	  ((atom trans) (merror (intl:gettext "changevar: second argument must not be an atom; found: ~M") trans))
+	  ((null (atom nvar)) (merror (intl:gettext "changevar: third argument must be an atom; found: ~M") nvar))
+	  ((null (atom ovar)) (merror (intl:gettext "changevar: fourth argument must be an atom; found: ~M") ovar)))
     (setq tfun (solvable (setq trans (meqhk trans)) ovar))
     (changevar expr)))
 
@@ -321,7 +336,7 @@
   (let (*roots *failures)
     (solve l var 1)
     (cond (*roots ($rhs (car *roots)))
-	  (errswitch (merror "Unable to solve for ~M" var))
+	  (errswitch (merror (intl:gettext "changevar: failed to solve for ~M in ~M") var l))
 	  (t nil))))
 
 (defun changevar (expr)
@@ -334,14 +349,14 @@
 				      (sdiff trans ovar))))))
 	     (cond ((and (member (caar expr) '(%sum %product) :test #'eq)
 			 (not (equal deriv 1)))
-		    (merror "Illegal change in summation or product"))
+		    (merror (intl:gettext "changevar: illegal change in summation or product")))
 		   ((setq nfun ($radcan	;NIL IF KERNSUBST FAILS
 				(if tfun
 				    (mul (maxima-substitute tfun ovar (cadr expr))
 					 deriv)
 				    (kernsubst ($ratsimp (mul (cadr expr)
 							      deriv))
-					       trans ovar)))) 
+					       trans ovar))))
 		    (cond ;; DEFINITE INTEGRAL,SUMMATION, OR PRODUCT
 		      ((cdddr expr)
 		       (or invfun (setq invfun (solvable trans nvar t)))
@@ -355,7 +370,7 @@
 				     '$minus)))
 		      (t		;INDEFINITE INTEGRAL
 		       (list '(%integrate) nfun nvar))))
-		   (t expr)))))) 
+		   (t expr))))))
 
 (defun kernsubst (expr form ovar)
   (let (varlist genvar nvarlist)
@@ -367,22 +382,22 @@
 	(prog2 (setq expr (ratrep* expr)
 		     varlist nvarlist)
 	    (rdis (cdr expr))))))
-	  
-(declare-top (special $listconstvars facfun)) 
+
+(declare-top (special $listconstvars facfun))
 
 (defmfun $factorsum (e)
-  (factorsum0 e '$factor)) 
+  (factorsum0 e '$factor))
 
 (defmfun $gfactorsum (e)
-  (factorsum0 e '$gfactor)) 
+  (factorsum0 e '$gfactor))
 
-(defun factorsum0 (e facfun) 
+(defun factorsum0 (e facfun)
   (cond ((mplusp (setq e (funcall facfun e)))
 	 (factorsum1 (cdr e)))
-	(t (factorsum2 e)))) 
+	(t (factorsum2 e))))
 
-(defun factorsum1 (e) 
-  (prog (f lv llv lex cl lt c) 
+(defun factorsum1 (e)
+  (prog (f lv llv lex cl lt c)
    loop (setq f (car e))
    (setq lv (cdr ($showratvars f)))
    (cond ((null lv) (setq cl (cons f cl)) (go skip)))
@@ -417,41 +432,41 @@
 		       (rplaca q (cons (dcon f) (car q)))
 		       (return (setq f nil)))))
 	      (and f
-		   (setq lv (cons c lv) 
+		   (setq lv (cons c lv)
 			 lt (cons (ncons (dcon f)) lt))))))
    (setq lex
-	 (mapcar #'(lambda (s q) 
+	 (mapcar #'(lambda (s q)
 		     (simptimes (list '(mtimes) s
 				      (cond ((cdr q)
 					     (cons '(mplus) q))
 					    (t (car q))))
 				1 nil))
 		 lv lt))
-   (return (simplus (cons '(mplus) (nconc cl lex llv)) 1 nil)))) 
+   (return (simplus (cons '(mplus) (nconc cl lex llv)) 1 nil))))
 
-(defun dcon (mt) 
-  (cond ((cdddr mt) (cons (car mt) (cddr mt))) (t (caddr mt)))) 
+(defun dcon (mt)
+  (cond ((cdddr mt) (cons (car mt) (cddr mt))) (t (caddr mt))))
 
-(defun factorsum2 (e) 
+(defun factorsum2 (e)
   (cond ((not (mtimesp e)) e)
 	(t (cons '(mtimes)
-		 (mapcar #'(lambda (f) 
+		 (mapcar #'(lambda (f)
 			     (cond ((mplusp f)
 				    (factorsum1 (cdr f)))
 				   (t f)))
-			 (cdr e)))))) 
+			 (cdr e))))))
 
 (declare-top (special $combineflag))
 
 (defmvar $combineflag t)
 
-(defmfun $combine (e) 
+(defmfun $combine (e)
   (cond ((or (atom e) (eq (caar e) 'rat)) e)
 	((eq (caar e) 'mplus) (combine (cdr e)))
-	(t (recur-apply #'$combine e)))) 
+	(t (recur-apply #'$combine e))))
 
-(defun combine (e) 
-  (prog (term r ld sw nnu d ln xl) 
+(defun combine (e)
+  (prog (term r ld sw nnu d ln xl)
    again(setq term (car e) e (cdr e))
    (when (or (not (or (ratnump term) (mtimesp term) (mexptp term)))
 	     (equal (setq d ($denom term)) 1))
@@ -470,8 +485,8 @@
    end  (and e (go again))
    (and xl (setq xl (cond ((cdr xl) ($xthru (addn xl t)))
 			  (t (car xl)))))
-   (mapc 
-    #'(lambda (nu de) 
+   (mapc
+    #'(lambda (nu de)
 	(setq r (cons (mul2 (addn nu nil) (power* de -1)) r)))
     ln ld)
    (return (addn (if xl (cons xl r) r) nil))))
@@ -479,7 +494,7 @@
 (defmfun $factorout (e &rest vl)
   (prog (el fl cl l f x)
      (when (null vl)
-       (merror "`factorout' called on only one argument"))
+       (merror (intl:gettext "factorout: at least two arguments required.")))
      (unless (mplusp e)
        (return e))
      (or (null vl) (mplusp e) (return e))
