@@ -78,22 +78,22 @@ sin(y)*(10.0+6*cos(x)),
     ;; adaptive-plotting will do.
     ((mlist) $adapt_depth 5)
     ((mlist) $gnuplot_preamble "")
-    ((mlist) $gnuplot_default_term_command "set term pop")
+    ((mlist) $gnuplot_default_term_command ,(if (string= *autoconf-win32* "true") "set term windows" "set term pop"))
     ((mlist) $gnuplot_dumb_term_command
      "set term dumb 79 22")
     ((mlist) $gnuplot_ps_term_command
      "set size 1.5, 1.5;set term postscript eps enhanced color solid 24")
     ((mlist) $plot_realpart nil)))
 
-;; $plot_realpart option is false by default but *plot-realpart* is true because coerce-float-fun
-;; is used outside of plot package too.
+;; $plot_realpart option is false by default but *plot-realpart* is true
+;; because coerce-float-fun is used outside of plot package too.
 (defvar *plot-realpart* t)
 
 (defun maybe-realpart (x)
   (if *plot-realpart*
       ($realpart x)
-      (if (eq 0 ($imagpart x))
-          x
+      (if (zerop1 ($imagpart x))
+          ($realpart x)
           nil)))
 
 (defvar *missing-data-indicator* "NaN")
@@ -551,21 +551,25 @@ sin(y)*(10.0+6*cos(x)),
 ;;   name of a simplifying function
 ;; EXPR is a Maxima lambda expression
 ;; EXPR is a general Maxima expression
-
-(defun coerce-float-fun (expr &optional lvars)
+;;
+;; %COERCE-FLOAT-FUN is the main internal routine for this.
+;; COERCE-FLOAT-FUN is the user interface for creating a function that
+;; returns floats.  COERCE-BFLOAT-FUN is the same, except bfloats are
+;; returned.
+(defun %coerce-float-fun (float-fun expr &optional lvars)
   (cond ((and (consp expr) (functionp expr))
          (let ((args (if lvars (cdr lvars) (list (gensym)))))
-           (coerce-lisp-function-or-lisp-lambda args expr)))
+           (coerce-lisp-function-or-lisp-lambda args expr :float-fun float-fun)))
         ;; expr is a string which names an operator
         ;; (e.g. "!" "+" or a user-defined operator)
         ((and (stringp expr) (getopr0 expr))
          (let ((a (if lvars lvars `((mlist) ,(gensym)))))
-           (coerce-float-fun `(($apply) ,(getopr0 expr) ,a) a)))
+           (%coerce-float-fun float-fun `(($apply) ,(getopr0 expr) ,a) a)))
         ((and (symbolp expr) (not (member expr lvars)) (not ($constantp expr)))
          (cond
            ((fboundp expr)
             (let ((args (if lvars (cdr lvars) (list (gensym)))))
-              (coerce-lisp-function-or-lisp-lambda args expr)))
+              (coerce-lisp-function-or-lisp-lambda args expr :float-fun float-fun)))
 
            ;; expr is name of a Maxima function defined by := or
            ;; define
@@ -573,7 +577,7 @@ sin(y)*(10.0+6*cos(x)),
             (let*
                 ((mexpr (mget expr 'mexpr))
                  (args (cdr (second mexpr))))
-              (coerce-maxima-function-or-maxima-lambda args expr)))
+              (coerce-maxima-function-or-maxima-lambda args expr :float-fun float-fun)))
 
            ((or
              ;; expr is the name of a function defined by defmspec
@@ -589,122 +593,150 @@ sin(y)*(10.0+6*cos(x)),
              ;; form
              (get ($verbify expr) 'operators))
             (let ((a (if lvars lvars `((mlist) ,(gensym)))))
-              (coerce-float-fun `(($apply) ,expr ,a) a)))
+              (%coerce-float-fun float-fun `(($apply) ,expr ,a) a)))
            (t
             (merror (intl:gettext "COERCE-FLOAT-FUN: no such Lisp or Maxima function: ~M") expr))))
 
-    ((and (consp expr) (eq (caar expr) 'lambda))
-     (let ((args (cdr (second expr))))
-       (coerce-maxima-function-or-maxima-lambda args expr)))
+	((and (consp expr) (eq (caar expr) 'lambda))
+	 (let ((args (cdr (second expr))))
+	   (coerce-maxima-function-or-maxima-lambda args expr :float-fun float-fun)))
 
         (t
          (let* ((vars (or lvars ($sort ($listofvars expr))))
-            (subscripted-vars ($sublist vars '((lambda) ((mlist) $x) ((mnot) (($atom) $x)))))
-            gensym-vars save-list-gensym subscripted-vars-save
-            subscripted-vars-mset subscripted-vars-restore)
+		(subscripted-vars ($sublist vars '((lambda) ((mlist) $x) ((mnot) (($atom) $x)))))
+		gensym-vars save-list-gensym subscripted-vars-save
+		subscripted-vars-mset subscripted-vars-restore)
 
-       ;; VARS and SUBSCRIPTED-VARS are Maxima lists.  Other lists are
-       ;; Lisp lists.
-       (when (cdr subscripted-vars)
-         (setq gensym-vars (mapcar #'(lambda (ign) (declare (ignore ign)) (gensym))
-                                   (cdr subscripted-vars)))
-         (mapcar #'(lambda (a b) (setq vars (subst b a vars :test 'equal)))
-                 (cdr subscripted-vars) gensym-vars)
+	   ;; VARS and SUBSCRIPTED-VARS are Maxima lists.  Other lists are
+	   ;; Lisp lists.
+	   (when (cdr subscripted-vars)
+	     (setq gensym-vars (mapcar #'(lambda (ign) (declare (ignore ign)) (gensym))
+				       (cdr subscripted-vars)))
+	     (mapcar #'(lambda (a b) (setq vars (subst b a vars :test 'equal)))
+		     (cdr subscripted-vars) gensym-vars)
 
-         ;; This stuff about saving and restoring array variables
-         ;; should go into MBINDING, and the lambda expression
-         ;; constructed below should call MBINDING.  (At present
-         ;; MBINDING barfs on array variables.)
-         (setq save-list-gensym (gensym))
-         (setq subscripted-vars-save
-               (mapcar #'(lambda (a) `(push (meval ',a) ,save-list-gensym))
-                       (cdr subscripted-vars)))
-         (setq subscripted-vars-mset
-               (mapcar #'(lambda (a b) `(mset ',a ,b))
-                       (cdr subscripted-vars) gensym-vars))
-         (setq subscripted-vars-restore
-               (mapcar #'(lambda (a) `(mset ',a (pop ,save-list-gensym)))
-                       (reverse (cdr subscripted-vars)))))
+	     ;; This stuff about saving and restoring array variables
+	     ;; should go into MBINDING, and the lambda expression
+	     ;; constructed below should call MBINDING.  (At present
+	     ;; MBINDING barfs on array variables.)
+	     (setq save-list-gensym (gensym))
+	     (setq subscripted-vars-save
+		   (mapcar #'(lambda (a) `(push (meval ',a) ,save-list-gensym))
+			   (cdr subscripted-vars)))
+	     (setq subscripted-vars-mset
+		   (mapcar #'(lambda (a b) `(mset ',a ,b))
+			   (cdr subscripted-vars) gensym-vars))
+	     (setq subscripted-vars-restore
+		   (mapcar #'(lambda (a) `(mset ',a (pop ,save-list-gensym)))
+			   (reverse (cdr subscripted-vars)))))
 
-       (coerce
-        `(lambda ,(cdr vars)
-           (declare (special ,@(cdr vars) errorsw))
+	   (coerce
+	    `(lambda ,(cdr vars)
+	       (declare (special ,@(cdr vars) errorsw))
 
-           ;; Nothing interpolated here when there are no subscripted
-           ;; variables.
-           ,@(if save-list-gensym `((declare (special ,save-list-gensym))))
+	       ;; Nothing interpolated here when there are no subscripted
+	       ;; variables.
+	       ,@(if save-list-gensym `((declare (special ,save-list-gensym))))
 
-           ;; Nothing interpolated here when there are no subscripted
-           ;; variables.
-           ,@(if (cdr subscripted-vars)
-                 `((progn (setq ,save-list-gensym nil)
-                          ,@(append subscripted-vars-save subscripted-vars-mset))))
+	       ;; Nothing interpolated here when there are no subscripted
+	       ;; variables.
+	       ,@(if (cdr subscripted-vars)
+		     `((progn (setq ,save-list-gensym nil)
+			      ,@(append subscripted-vars-save subscripted-vars-mset))))
 
-           (let (($ratprint nil) ($numer t) (nounsflag t)
-                 (errorsw t)
-                 (errcatch t))
-             (declare (special errcatch))
-             ;; Catch any errors from evaluating the
-             ;; function.  We're assuming that if an error
-             ;; is caught, the result is not a number.  We
-             ;; also assume that for such errors, it's
-             ;; because the function is not defined there,
-             ;; not because of some other maxima error.
-             ;;
-             ;; GCL 2.6.2 has handler-case but not quite ANSI yet. 
-             (let ((result
-                    #-gcl
-                     (handler-case 
-                         (catch 'errorsw
-                           ($float (maybe-realpart (meval* ',expr))))
-                       ;; Should we just catch all errors here?  It is
-                       ;; rather nice to only catch errors we care
-                       ;; about and let other errors fall through so
-                       ;; that we don't pretend to do something when
-                       ;; it is better to let the error through.
-                       (arithmetic-error () t)
-                       (maxima-$error () t))
-                     #+gcl
-                     (handler-case 
-                         (catch 'errorsw
-                           ($float (maybe-realpart (meval* ',expr))))
-                       (cl::error () t))
-                     ))
+	       (let (($ratprint nil)
+		     ;; We don't want to set $numer to T when coercing
+		     ;; to a bigfloat.  By doing so, things like
+		     ;; log(400)^400 get converted to double-floats,
+		     ;; which causes a double-float overflow.  But the
+		     ;; whole point of coercing to bfloat is to use
+		     ;; bfloats, not doubles.
+		     ;;
+		     ;; Perhaps we don't even need to do this for
+		     ;; double-floats?  It would be nice to remove
+		     ;; this.  For backward compatibility, we bind
+		     ;; numer to T if we're not trying to bfloat.
+		     ($numer ,(not (eq float-fun '$bfloat)))
+		     (*nounsflag* t)
+		     (errorsw t)
+		     (errcatch t))
+		 (declare (special errcatch))
+		 ;; Catch any errors from evaluating the
+		 ;; function.  We're assuming that if an error
+		 ;; is caught, the result is not a number.  We
+		 ;; also assume that for such errors, it's
+		 ;; because the function is not defined there,
+		 ;; not because of some other maxima error.
+		 ;;
+		 ;; GCL 2.6.2 has handler-case but not quite ANSI yet. 
+		 (let ((result
+			#-gcl
+			 (handler-case 
+			     (catch 'errorsw
+			       (,float-fun (maybe-realpart (meval* ',expr))))
+			   ;; Should we just catch all errors here?  It is
+			   ;; rather nice to only catch errors we care
+			   ;; about and let other errors fall through so
+			   ;; that we don't pretend to do something when
+			   ;; it is better to let the error through.
+			   (arithmetic-error () t)
+			   (maxima-$error () t))
+			 #+gcl
+			 (handler-case 
+			     (catch 'errorsw
+			       (,float-fun (maybe-realpart (meval* ',expr))))
+			   (cl::error () t))
+			 ))
 
-               ;; Nothing interpolated here when there are no
-               ;; subscripted variables.
-               ,@(if (cdr subscripted-vars) `((progn ,@subscripted-vars-restore)))
+		   ;; Nothing interpolated here when there are no
+		   ;; subscripted variables.
+		   ,@(if (cdr subscripted-vars) `((progn ,@subscripted-vars-restore)))
 
-               result)))
-        'function)))))
+		   result)))
+	    'function)))))
 
-(defun coerce-maxima-function-or-maxima-lambda (args expr)
+(defun coerce-float-fun (expr &optional lvars)
+  (%coerce-float-fun '$float expr lvars))
+
+(defun coerce-bfloat-fun (expr &optional lvars)
+  (%coerce-float-fun '$bfloat expr lvars))
+
+(defun coerce-maxima-function-or-maxima-lambda (args expr &key (float-fun '$float))
   (let ((gensym-args (loop for x in args collect (gensym))))
     (coerce
       `(lambda ,gensym-args (declare (special ,@gensym-args))
          (let* (($ratprint nil)
                 ($numer t)
-                (nounsflag t)
-                (result (maybe-realpart (mapply ',expr (list ,@gensym-args) t))))
-           ;; Just always try to convert the result to a float, which
-           ;; handles things like $%pi.  See also BUG #2880115
-           ;; http://sourceforge.net/tracker/?func=detail&atid=104933&aid=2880115&group_id=4933
-           ($float result)))
+                (*nounsflag* t)
+		(errorsw t)
+		(errcatch t))
+	   (declare (special errcatch))
+	   ;; Just always try to convert the result to a float,
+	   ;; which handles things like $%pi.  See also BUG
+	   ;; #2880115
+	   ;; http://sourceforge.net/tracker/?func=detail&atid=104933&aid=2880115&group_id=4933
+	   ;;
+	   ;; Should we use HANDLER-CASE like we do above in
+	   ;; %coerce-float-fun?  Seems not necessary for what we want
+	   ;; to do.
+	   (catch 'errorsw
+	     (,float-fun
+	      (maybe-realpart (mapply ',expr (list ,@gensym-args) t))))))
       'function)))
 
 ;; Same as above, but call APPLY instead of MAPPLY.
 
-(defun coerce-lisp-function-or-lisp-lambda (args expr)
+(defun coerce-lisp-function-or-lisp-lambda (args expr &key (float-fun '$float))
   (let ((gensym-args (loop for x in args collect (gensym))))
     (coerce
       `(lambda ,gensym-args (declare (special ,@gensym-args))
          (let* (($ratprint nil)
                 ($numer t)
-                (nounsflag t)
+                (*nounsflag* t)
                 (result (maybe-realpart (apply ',expr (list ,@gensym-args)))))
            ;; Always use $float.  See comment for
            ;; coerce-maxima-function-ormaxima-lambda above.
-           ($float result)))
+           (,float-fun result)))
       'function)))
 
 (defmacro zval (points verts i) `(aref ,points (+ 2 (* 3 (aref ,verts ,i)))))
@@ -947,7 +979,7 @@ sin(y)*(10.0+6*cos(x)),
            ;; 100*flonum-epsilon is ok.
            (let ((diff (- (abs quad)
                           (* eps (- quad-b (min f-a f-a1 f-b f-b1 f-c)))))
-                 (delta (* 100 flonum-epsilon)))
+                 (delta (* 150 flonum-epsilon)))
              (<= diff delta))))
         (t
          ;; Something is not a number, so assume it's not smooth enough.
@@ -1603,25 +1635,26 @@ output-file))
   (and (symbolp x) (char= (char (symbol-value x) 0) #\$)))
 
 
-(defun $tcl_output  (lis i &optional (skip 2))
-  (or (typep i 'fixnum) (error "~a should be an integer" i ))
-  ($listp_check 'list lis )
+(defun $tcl_output (lis i &optional (skip 2))
+  (when (not (typep i 'fixnum))
+    (merror
+      (intl:gettext "tcl_ouput: second argument must be an integer; found ~M")
+                    i))
+  (when (not ($listp lis))
+    (merror
+      (intl:gettext "tcl_output: first argument must be a list; found ~M") lis))
   (format *standard-output* "~% {")
   (cond (($listp (second lis))
          (loop for v in lis
                 do
-                (format *standard-output* "~,10g " (nth i v)))
-         )
+                (format *standard-output* "~,10g " (nth i v))))
         (t
          (setq lis (nthcdr i lis))
          (loop  with v = lis  while v
                  do
                  (format *standard-output* "~,10g " (car v))
-                 (setq v (nthcdr skip v))
-                 )
-         ))
-  (format *standard-output* "~% }")
-  )
+                 (setq v (nthcdr skip v)))))
+  (format *standard-output* "~% }"))
 
 
 (defun tcl-output-list ( st lis )
@@ -1813,6 +1846,7 @@ output-file))
      lvars trans xrange yrange *original-points*
      functions exprn domain tem ($plot_options $plot_options)
      ($in_netmath $in_netmath) features
+     (*plot-realpart* *plot-realpart*)
      gnuplot-term gnuplot-out-file file titles (output-file "")
      (usage (intl:gettext
 "plot3d: Usage.
@@ -1947,13 +1981,17 @@ Several functions depending on the two variables v1 and v2:
   ;; parse the options given to plot3d
   (setq features (plot-options-parser options features))
   (setq tem ($get_plot_option '$transform_xy 2))
+  (setq *plot-realpart* ($get_plot_option '$plot_realpart 2))
   
   ;; set up the labels for the axes
   (when (or (null (getf features :box)) (first (getf features :box)))
     (if (and (getf features :xvar) (getf features :yvar) (null tem))
 	(progn
-	  (setf (getf features :xlabel) (ensure-string (getf features :xvar)))
-	  (setf (getf features :ylabel) (ensure-string (getf features :yvar))))
+	  ;; Don't set xlabel (ylabel) if the user specified one.
+	  (unless (getf features :xlabel)
+	    (setf (getf features :xlabel) (ensure-string (getf features :xvar))))
+	  (unless (getf features :ylabel)
+	    (setf (getf features :ylabel) (ensure-string (getf features :yvar)))))
 	(progn
 	  (setf (getf features :xlabel) "x")
 	  (setf (getf features :ylabel) "y")))
@@ -2025,8 +2063,8 @@ Several functions depending on the two variables v1 and v2:
                         (third lvars)
                         (rest ($listofvars (mfuncall fun (second lvars)
                                                      (third lvars))))))
-                  (merror
-                   (intl:gettext "plot3d: expected <expr. of v1 and v2>, [v1, min, max], [v2, min, max]")))))
+                  (mtell (intl:gettext "plot3d: expected <expr. of v1 and v2>, [v1, min, max], [v2, min, max]~%"))
+                  (mtell (intl:gettext "plot3d: keep going and hope for the best.~%")))))
           (let* ((pl
                   (draw3d
                    fun (third xrange) (fourth xrange) (third yrange)
