@@ -308,35 +308,32 @@
   (prog (maxexp i l *p factors factor errrjfflag)
      (setq maxexp (quotient (cadr p) 2))
      (setq i 1)
-     a    (cond ((> i maxexp) (return (cons p factors))))
-     (setq l (p1 (reverse ((lambda (p i $factorflag)
-			     (pfactor2 p i))
-			   p
-			   i
-			   t))))
-     b    (cond ((null l) (go d)))
+     a    (when (> i maxexp) (return (cons p factors)))
+     (setq l (p1 (reverse (let ((p p) (i i) ($factorflag t))
+			    (pfactor2 p i)))))
+     b    (when (null l) (go d))
      (setq *l (car l))
      (setq *p (car p))
      (setq errrjfflag t)
      (setq factor (errset (pinterpolate *l *p) nil))
      (setq errrjfflag nil)
      (setq l (cdr l))
-     (cond ((atom factor) (go b))
-	   (t (setq factor (car factor))))
-     (cond ((or (pcoefp factor)
-		(not (eqn (car p) (car factor)))
-		(not (pzerop (prem p factor))))
-	    (go b)))
+     (if (atom factor)
+	 (go b)
+	 (setq factor (car factor)))
+     (when (or (pcoefp factor)
+	       (not (eqn (car p) (car factor)))
+	       (not (pzerop (prem p factor))))
+       (go b))
      (cond (modulus (pmonicize (cdr factor)))
 	   ((pminusp factor) (setq factor (pminus factor))))
      (setq p (pquotient p factor))
      (setq maxexp (quotient (cadr p) 2))
      (setq factors (cons factor factors))
-     (cond ((or (eqn p 1) (eqn p -1)) (return factors)))
+     (when (or (eqn p 1) (eqn p -1)) (return factors))
      (go a)
-     d    (setq i (1+ i))
-     (go a)
-     ))
+     d    (incf i)
+     (go a)))
 
 (defun pfactor2 (p i)
   (cond ((< i 0) nil)
@@ -438,7 +435,7 @@
 	      (equal (car x) (car y)) (equal (caar x) 'mrat)
 	      (equal (cddr x) 1) (equal (cddr y) 1))
 	 (cons (car x)(cons (fptimes (cadr x)(cadr y))1)))
-	(t (merror "Use `fasttimes' only on CRE polynomials with same varlists"))))
+	(t (merror (intl:gettext "fasttimes: arguments must be CRE polynomials with same variables.")))))
 
 (defun fptimes (x y)
   (cond ((or (pzerop x) (pzerop y)) (pzero))
@@ -485,7 +482,7 @@
 
 (defmfun $ratweight (&rest args)
   (when (oddp (length args))
-    (merror "`ratweight' takes an even number of arguments."))
+    (merror (intl:gettext "ratweight: number of arguments must be a multiple of 2.")))
   (do ((l args (cddr l)))
       ((null l))
     (rplacd (or (assoc (first l) *ratweights :test #'equal)
@@ -527,18 +524,18 @@
 
 
 (defun wtptimes2 (y)
-  (cond ((null y) nil)
-	(t ((lambda (ii) (declare (fixnum ii))
-		    (cond ((> ii $ratwtlvl) (wtptimes2 (cddr y)))
-			  (t (pcoefadd (+ (car *x*) (car y))
-				       (wtptimes (cadr *x*) (cadr y) ii)
-				       (wtptimes2 (cddr y))))))
-	    (+ (* xweight (+ (car *x*) (car y))) wtsofar)))))
+  (if (null y)
+      nil
+      (let ((ii (+ (* xweight (+ (car *x*) (car y))) wtsofar)))
+	(if (> ii $ratwtlvl)
+	    (wtptimes2 (cddr y))
+	    (pcoefadd (+ (car *x*) (car y))
+		      (wtptimes (cadr *x*) (cadr y) ii)
+		      (wtptimes2 (cddr y)))))))
 
 (defun wtptimes3 (y)
   (prog ((e 0) u c)
-     (declare (fixnum e) (special v))
-
+     (declare (special v))
      a1   (cond ((null y) (return nil)))
      (setq e (+ (car *x*) (car y)))
      (setq c (wtptimes (cadr y) (cadr *x*) (+ wtsofar (* xweight e))))
@@ -625,7 +622,7 @@
 				    ans)))))
      (go a)))
 
-(declare-top (special y rischpf genvar $savefactors checkfactors w
+(declare-top (special y genvar $savefactors checkfactors w
 		      exp var x $factorflag $ratfac
 		      $keepfloat ratform rootfactor
 		      wholepart parnumer varlist n))
@@ -707,7 +704,7 @@
     (or (every #'(lambda (exp)
 		     (or (alike1 x exp) (free exp x)))
 		 varlist)
-	(merror "`ratdiff' variable is embedded in kernel"))
+	(merror (intl:gettext "ratdiff: first argument must be a polynomial in ~M; found: ~M") x p))
     (setq p (ratf p))
     (setq x (caadr (ratf x)))
     (setq p (cons (car p) (ratderivative (cdr p) x)))
